(*---------------------------------------------------------------------------*
  INTERFACE  iom_pipe.mli

  Copyright (c) 2006, James H. Woodyatt
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGE. 
 *---------------------------------------------------------------------------*)

(** Pipes, reading and writing. *)

(** {6 Overview}

    I/O stream gadgets are defined for reading and writing to file descriptors
    created with {!Unix.pipe}.
*)

(** A gadget for reading from a pipe. *)
class reader:
    k:Iom_gadget.kernel -> fd:Unix.file_descr ->
    limits:Iom_octet_stream.limits ->
    (Iom_octet_stream.fragment, Iom_stream.flowcontrol, Iom_stream.failed)
        Iom_stream.ipad ->
    [Iom_stream.flowcontrol, Iom_stream.failed] Iom_reactor.reader

(** A gadget for writing to a pipe. *)
class writer:
    k:Iom_gadget.kernel -> fd:Unix.file_descr ->
    (Iom_octet_stream.fragment, Iom_stream.stop, Iom_stream.flownotify)
        Iom_stream.opad ->
    [Iom_octet_stream.fragment, Iom_stream.stop, Iom_stream.flownotify]
        Iom_reactor.writer

(** Evaluate [create ~limits k] to obtain the reader and writer fixes for the
    I/O stream gadgets connected to each end of a new pipe.
*)
val create:
    limits:Iom_octet_stream.limits -> Iom_gadget.kernel ->
    (Iom_reactor.reader_fix * Iom_reactor.writer_fix) Iom_gadget.t

(*--- End of File [ iom_pipe.mli ] ---*)
